import { defineComponent, getCurrentInstance, computed, ref, watch, onMounted, onBeforeUnmount, createVNode } from 'vue';
import { findDOMNode, getComponent } from '../_util/props-util/index.js';
import '../_util/css/index.js';
import { getScrollParent } from '../_util/Dom/getScrollParent.js';
import formatLength from '../_util/css/format-length.js';

const preCls = "x-image";
var Image = defineComponent({
  name: preCls,
  props: {
    src: String,
    width: [String, Number],
    height: [String, Number],
    fit: String,
    alt: String,
    placeholder: [Boolean, Object, String],
    fallback: String,
    lazy: Boolean
  },
  emits: ["load", "error"],
  setup(props, {
    emit,
    slots
  }) {
    const {
      proxy: vm
    } = getCurrentInstance();
    const hasPlaceholder = computed(() => !!props.placeholder || !!slots.placeholder);
    const statusRef = ref();
    watch(() => props.src, () => {
      statusRef.value = hasPlaceholder.value ? "loading" : "normal";
    }, {
      immediate: true
    });
    const showRef = ref(!props.lazy);
    let io;
    onMounted(() => {
      if (!props.lazy)
        return;
      const el = findDOMNode(vm);
      io = new IntersectionObserver(([entry]) => {
        if (!entry.isIntersecting)
          return;
        showRef.value = true;
        io.disconnect();
        io = void 0;
      }, {
        root: getScrollParent(el)
      });
      io.observe(el);
    });
    onBeforeUnmount(() => {
      io == null ? void 0 : io.disconnect();
    });
    function onLoad(e) {
      statusRef.value = "normal";
      emit("load", e);
    }
    function onError(e) {
      statusRef.value = "error";
      emit("error", e);
    }
    function getImg() {
      if (!showRef.value)
        return;
      const imgProps = {
        class: `${preCls}_img`,
        style: {
          objectFit: props.fit
        },
        alt: props.alt,
        src: props.src,
        onLoad,
        onError
      };
      return createVNode("img", imgProps, null);
    }
    function getPlaceholder() {
      if (!hasPlaceholder.value)
        return;
      return createVNode("div", {
        "class": `${preCls}_placeholder`
      }, [props.placeholder !== true ? getComponent(vm, "placeholder") : "\u52A0\u8F7D\u4E2D\u2026\u2026"]);
    }
    function getError() {
      var _a;
      if (props.fallback) {
        return createVNode("img", {
          "class": `${preCls}_error`,
          "src": props.fallback
        }, null);
      } else {
        return createVNode("div", {
          "class": `${preCls}_error`
        }, [((_a = slots.error) == null ? void 0 : _a.call(slots)) || "\u2517( T\uFE4FT )\u251B"]);
      }
    }
    return () => {
      const status = statusRef.value;
      const style = {
        width: formatLength(props.width),
        height: formatLength(props.height)
      };
      return createVNode("div", {
        "class": preCls,
        "style": style
      }, [status !== "error" && getImg(), status === "loading" ? getPlaceholder() : status === "error" ? getError() : void 0]);
    };
  }
});

export { Image as default };
