import { defineComponent, getCurrentInstance, ref, computed, createVNode, Fragment, mergeProps, withDirectives, vShow } from 'vue';
import { isArray } from '@vue/shared';
import { castArray } from '../_util/_.js';
import { getComponent } from '../_util/props-util/index.js';
import '../_util/hooks/index.js';
import isValid from '../_util/isValid.js';
import '../icon/index.js';
import inputProps from './inputProps.js';
import { calcTextareaHeight } from './utils.js';
import useNamespace from '../_util/hooks/useNameSpace2.js';
import useMemo from '../_util/hooks/useMemo.js';
import Icon from '../icon/Icon.js';

const prefixCls = "x-input";
var Input = defineComponent({
  name: prefixCls,
  props: inputProps,
  emits: ["update:value", "change", "input", "pressEnter", "keydown", "focus", "blur"],
  setup(props, {
    emit,
    expose
  }) {
    const ns = useNamespace(prefixCls);
    const vm = getCurrentInstance().proxy;
    const inputRef = ref();
    const focusedRef = ref();
    const valRef = useMemo(() => props.pair ? castArray(props.value) : props.value);
    const showClearRef = computed(() => castArray(valRef.value).some(isValid) && !props.disabled);
    function onInput(e) {
      var _a, _b;
      const value = (_b = (_a = e == null ? void 0 : e.target) == null ? void 0 : _a.value) != null ? _b : "";
      if (props.pair) {
        if (!isArray(valRef.value)) {
          throw new Error("valRef.value is not a Array type");
        }
        if (e) {
          if (e.target == inputRef.value)
            valRef.value[0] = value;
          else
            valRef.value[1] = value;
        } else {
          valRef.value = ["", ""];
        }
      } else {
        valRef.value = value;
      }
      emit("update:value", valRef.value);
      emit("input", valRef.value);
    }
    function onChange(e) {
      emit("change", valRef.value);
    }
    function onFocus(e) {
      focusedRef.value = true;
      emit("focus", e);
    }
    function onBlur(e) {
      focusedRef.value = false;
      emit("blur", e);
    }
    function onKeydown(e) {
      if (e.keyCode === 13) {
        emit("pressEnter", e);
      }
      emit("keydown", e);
    }
    function onClearDown() {
      window.requestAnimationFrame(focus);
    }
    function onClear(e) {
      e.stopPropagation();
      inputRef.value.value = "";
      onInput();
      focus();
    }
    function focus() {
      inputRef.value.focus();
    }
    function blur() {
      inputRef.value.blur();
    }
    expose({
      focus,
      blur,
      get value() {
        return valRef.value;
      }
    });
    const on = {
      onInput,
      onChange,
      onKeydown,
      onFocus,
      onBlur
    };
    function renderInput() {
      const {
        disabled,
        type
      } = props;
      const placeholder = castArray(props.placeholder);
      const value = castArray(valRef.value);
      const addon1Node = getComponent(vm, "addonBefore");
      const addon2Node = getComponent(vm, "addonAfter");
      const prefixNode = getComponent(vm, "prefix");
      const suffixNode = getComponent(vm, "suffix");
      return createVNode(Fragment, null, [addon1Node && createVNode("span", {
        "class": ns.e("addon")
      }, [addon1Node]), prefixNode && createVNode("span", {
        "class": ns.e("prefix")
      }, [prefixNode]), createVNode("input", mergeProps({
        "class": ns.e("input"),
        "ref": inputRef,
        "value": value[0],
        "type": type,
        "disabled": disabled,
        "placeholder": placeholder[0]
      }, on), null), props.pair && createVNode("span", {
        "class": ns.e("separator")
      }, [props.separator]), props.pair && createVNode("input", mergeProps({
        "class": ns.e("input"),
        "value": value[1],
        "type": type,
        "disabled": disabled,
        "placeholder": placeholder[1] || placeholder[0]
      }, on), null), suffixNode && createVNode("span", {
        "class": ns.e("suffix")
      }, [suffixNode]), addon2Node && createVNode("span", {
        "class": ns.e("addon")
      }, [addon2Node])]);
    }
    function renderTextarea() {
      const {
        disabled,
        placeholder
      } = props;
      const style = calcTextareaHeight(inputRef.value, props.autosize);
      return createVNode("textarea", mergeProps({
        "class": ns.e("input"),
        "ref": inputRef,
        "style": style,
        "value": valRef.value,
        "disabled": disabled,
        "placeholder": placeholder
      }, on), null);
    }
    return () => {
      const {
        disabled,
        clearable
      } = props;
      const cls = [ns.b(), ns.m(props.type), props.pair && ns.m("pair"), disabled && ns.m("disabled"), focusedRef.value && ns.m(`focus`)];
      return createVNode("div", {
        "class": cls
      }, [props.type !== "textarea" ? renderInput() : renderTextarea(), clearable && withDirectives(createVNode("span", {
        "class": `${prefixCls}_clear`,
        "onMousedown": onClearDown,
        "onClick": onClear
      }, [createVNode(Icon, {
        "type": "cross"
      }, null)]), [[vShow, showClearRef.value]])]);
    };
  }
});

export { Input as default };
