import { defineComponent, computed, provide, createVNode, mergeProps, withDirectives, vShow } from 'vue';
import { useMenu, createItem } from './utils.js';
import { omit } from '../_util/_.js';
import '../trigger/index.js';
import MenuItemContent from './MenuItemContent.js';
import '../expand-transition/index.js';
import ExpandTransition from '../expand-transition/expand-transition.js';
import Trigger from '../trigger/Trigger.js';

const submenuProps = {
  label: [String, Object],
  value: [String, Number],
  children: Array,
  icon: [Object, Function],
  disabled: Boolean
};
const SubmenuContextKey = Symbol("submenu");
var Submenu = defineComponent({
  name: "x-submenu",
  props: submenuProps,
  setup(props) {
    const {
      paddingLeftRef,
      menuContext,
      horizontalRef,
      inlineRef,
      isActiveRef,
      disabledRef,
      isRoot
    } = useMenu(props);
    const collapsedRef = computed(() => !menuContext.expandedKeysRef.value.includes(props.value));
    function onClick() {
      if (disabledRef.value || menuContext.collapsedRef.value || !inlineRef.value)
        return;
      menuContext.onExpanded(props);
    }
    function onPopupVisibleChange(visible) {
      menuContext.onExpanded(props);
    }
    provide(SubmenuContextKey, {
      props,
      paddingLeftRef
    });
    return () => {
      const prefixCls = "x-submenu";
      const horizontal = horizontalRef.value;
      const inline = inlineRef.value;
      const menuCollapsed = menuContext.collapsedRef.value;
      const collapsed = collapsedRef.value;
      const contentNode = createVNode(MenuItemContent, mergeProps(omit(props, "children"), {
        "showArrow": !menuCollapsed || !isRoot,
        "collapsed": collapsed,
        "style": `paddingLeft:${paddingLeftRef.value}px`,
        "active": isActiveRef.value,
        "onClick": onClick
      }), null);
      const childrenNode = createVNode(ExpandTransition, null, {
        default: () => [withDirectives(createVNode("div", {
          "class": `${prefixCls}_children`,
          "role": "menu"
        }, [props.children.map(createItem)]), [[vShow, !collapsed]])]
      });
      const popupNode = createVNode("div", {
        "class": `${prefixCls}_children`,
        "role": "menu"
      }, [props.children.map(createItem)]);
      return createVNode(Trigger, {
        "prefixCls": prefixCls,
        "disabled": disabledRef.value || (!(horizontal || menuCollapsed) ? inline : false),
        "action": "hover",
        "popup": popupNode,
        "popupVisible": !collapsed,
        "onPopupVisibleChange": onPopupVisibleChange,
        "popupClass": `${prefixCls}-popup`,
        "popupPlacement": horizontal && isRoot ? "bottom" : "rightTop"
      }, {
        default: () => [createVNode("div", {
          "class": prefixCls,
          "role": "menuitem",
          "aria-expanded": !collapsed,
          "aria-disabled": disabledRef.value
        }, [contentNode, !(menuCollapsed || horizontal || !inline) && childrenNode])]
      });
    };
  }
});

export { SubmenuContextKey, Submenu as default, submenuProps };
