import { inject, computed, h } from 'vue';
import { MenuContextKey } from './Menu.js';
import MenuItem from './MenuItem.js';
import MenuItemGroup from './MenuItemGroup.js';
import Submenu, { SubmenuContextKey } from './Submenu.js';

function useMenu(props) {
  const menuContext = inject(MenuContextKey, void 0);
  const submenuContext = inject(SubmenuContextKey, void 0);
  const { props: menuProps } = menuContext;
  const isRoot = menuContext && !submenuContext;
  const horizontalRef = computed(() => {
    return menuProps.mode === "horizontal";
  });
  const inlineRef = computed(() => {
    return menuProps.mode === "inline";
  });
  const indentRef = computed(() => {
    return horizontalRef.value ? 0 : menuProps.indent;
  });
  const parentPaddingLeftRef = computed(() => {
    if (menuContext.collapsedRef.value)
      return 0;
    if (isRoot)
      return 0;
    if (submenuContext) {
      return submenuContext.paddingLeftRef.value;
    }
  });
  const paddingLeftRef = computed(() => {
    if (horizontalRef.value)
      return void 0;
    if (!inlineRef.value)
      return void 0;
    return parentPaddingLeftRef.value + indentRef.value;
  });
  const disabledRef = computed(() => {
    return menuContext.props.disabled || (submenuContext == null ? void 0 : submenuContext.props.disabled) || props.disabled;
  });
  const isActiveRef = computed(() => {
    return menuContext.activePathRef.value.includes(props.value);
  });
  return {
    menuContext,
    submenuContext,
    horizontalRef,
    inlineRef,
    indentRef,
    parentPaddingLeftRef,
    paddingLeftRef,
    disabledRef,
    isActiveRef,
    isRoot
  };
}
function createItem(node) {
  if (node.children) {
    if (node.type === "group") {
      return h(MenuItemGroup, node);
    } else {
      return h(Submenu, node);
    }
  } else {
    return h(MenuItem, node);
  }
}
function traverseTree(tree, cb) {
  tree == null ? void 0 : tree.forEach((opt) => {
    if (opt.children)
      traverseTree(opt.children, cb);
    cb(opt);
  });
}
function getNonLeafKeys(tree) {
  const keys = [];
  traverseTree(tree, (opt) => keys.push(opt.value));
  return keys;
}
function treeFindPath(tree, func, path = []) {
  for (const item of tree) {
    path.push(item.value);
    if (func(item))
      return path;
    if (item.children) {
      const ret = treeFindPath(item.children, func, path);
      if (ret.length)
        return ret;
    }
    path.pop();
  }
  return [];
}

export { createItem, getNonLeafKeys, traverseTree, treeFindPath, useMenu };
