import { defineComponent, ref, computed, watchPostEffect, createVNode } from 'vue';
import { keyBy, castArray } from '../_util/_.js';
import '../_util/hooks/index.js';
import { normalizeOptions } from '../_util/normalizeOptions.js';
import SelectTrigger from './SelectTrigger.js';
import OptionList from './OptionList.js';
import Selector from './Selector/index.js';
import useSelectTriggerControl from './hooks/useSelectTriggerControl.js';
import '../icon/index.js';
import useMemo from '../_util/hooks/useMemo.js';
import Icon from '../icon/Icon.js';

const prefixCls = "x-select";
var Select = defineComponent({
  name: prefixCls,
  props: {
    value: [String, Number, Array],
    options: Array,
    multiple: Boolean,
    disabled: Boolean,
    showArrow: {
      default: true
    },
    clearable: Boolean,
    virtualScroll: Boolean,
    placeholder: {
      default: "\u8BF7\u9009\u62E9"
    },
    filterable: Boolean,
    filter: Function
  },
  slots: ["default", "label"],
  emits: ["update:value", "change", "select", "deselect", "search", "focus", "blur"],
  setup(props, {
    emit,
    expose,
    slots
  }) {
    const containerRef = ref();
    const popupWidth = ref(0);
    const inputRef = ref();
    const popupRef = ref();
    const _options = computed(() => {
      var _a;
      return normalizeOptions((_a = props.options) != null ? _a : []);
    });
    const options = computed(() => {
      const searchVal = search.value;
      if (props.filterable && searchVal) {
        const filter = props.filter || ((str, option) => option.label.toString().includes(str));
        return _options.value.filter((opt) => filter(searchVal, opt));
      }
      return _options.value;
    });
    const optionsBy = computed(() => keyBy(props.options, "value"));
    const values = useMemo(() => castArray(props.value));
    const selected = computed(() => values.value.map((e) => optionsBy.value[e]));
    const search = ref("");
    const valuesSet = computed(() => new Set(values.value));
    function setValue(arr) {
      const value = props.multiple ? [...arr] : arr[0];
      console.log(value);
      emit("update:value", value);
      emit("change", value);
    }
    function setSearchValue(text) {
      search.value = text;
      emit("search", text);
    }
    const opened = ref(false);
    function onToggleOpen(boo) {
      if (props.disabled)
        return;
      if (boo === void 0) {
        opened.value = !opened.value;
      } else {
        opened.value = boo;
      }
    }
    useSelectTriggerControl([containerRef, popupRef], onToggleOpen);
    function onSelect(option) {
      const value = option.value;
      if (!props.multiple) {
        values.value = [value];
        emit("select", option.value, option);
      } else {
        const index = values.value.indexOf(option.value);
        if (~index) {
          values.value.splice(index, 1);
          emit("deselect", value, option);
        } else {
          values.value.push(value);
          emit("select", option.value, option);
        }
      }
      search.value = "";
      const newVal = props.multiple ? [...values.value] : values.value[0];
      emit("update:value", newVal);
      emit("change", newVal, option);
      if (!props.multiple) {
        onToggleOpen(false);
      }
    }
    function onClear() {
      setValue([]);
      setSearchValue("");
    }
    function onSearch(text) {
      setSearchValue(text);
    }
    function onMenuMousedown(e) {
      e.preventDefault();
    }
    const focusRef = ref(false);
    function onFocus(e) {
      if (props.disabled)
        return;
      focusRef.value = true;
      setSearchValue("");
      emit("focus", e);
    }
    function onBlur(e) {
      focusRef.value = false;
      if (props.disabled)
        return;
      setSearchValue("");
      emit("blur", e);
    }
    function focus() {
      inputRef.value.focus();
    }
    function blur() {
      inputRef.value.blur();
    }
    expose({
      focus,
      blur
    });
    watchPostEffect(() => {
      if (!opened.value)
        return;
      if (!containerRef.value)
        return;
      popupWidth.value = containerRef.value.offsetWidth;
    });
    return () => {
      const {
        clearable,
        showArrow,
        disabled
      } = props;
      const vals = values.value;
      const popup = createVNode(OptionList, {
        "prefixCls": prefixCls,
        "ref": popupRef,
        "values": valuesSet.value,
        "options": options.value,
        "virtualScroll": props.virtualScroll,
        "style": "max-height: 500px",
        "onSelect": onSelect,
        "onMousedown": onMenuMousedown
      }, {
        label: slots.label
      });
      const triggerProps = {
        prefixCls,
        popup,
        popupStyle: `width: ${popupWidth.value}px`,
        visible: opened.value,
        disabled: props.disabled,
        filterable: props.filterable
      };
      const cls = [prefixCls, {
        [`${prefixCls}-multiple`]: props.multiple,
        [`${prefixCls}-single`]: !props.multiple,
        [`${prefixCls}-filterable`]: props.filterable,
        [`${prefixCls}-focus`]: focusRef.value,
        [`${prefixCls}-disabled`]: disabled
      }];
      const clearNode = !disabled && clearable && vals.length ? createVNode(Icon, {
        "class": `${prefixCls}-clear`,
        "onClick": onClear,
        "type": "cross"
      }, null) : void 0;
      const arrowNode = !disabled && !clearNode && showArrow ? createVNode(Icon, {
        "class": `${prefixCls}-arrow`,
        "type": "arrow-down"
      }, null) : void 0;
      return createVNode("div", {
        "ref": containerRef,
        "class": cls
      }, [createVNode(SelectTrigger, triggerProps, {
        default: () => [createVNode(Selector, {
          "prefixCls": prefixCls,
          "values": selected.value,
          "multiple": props.multiple,
          "searchValue": search.value,
          "filterable": props.filterable,
          "showArrow": props.showArrow,
          "clearable": props.clearable,
          "disabled": props.disabled,
          "placeholder": props.placeholder,
          "inputRef": inputRef,
          "onSelect": onSelect,
          "onSearch": onSearch,
          "onToggleOpen": onToggleOpen,
          "onFocus": onFocus,
          "onBlur": onBlur
        }, null)]
      }), clearNode, arrowNode]);
    };
  }
});

export { Select as default };
